@@ -1,5 +1,5 @@
 /* Definitions of target machine for GCC for IA-32.
-   Copyright (C) 1988-2013 Free Software Foundation, Inc.
+   Copyright (C) 1988-2015 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -40,70 +40,73 @@ extern void ix86_output_addr_diff_elt (FILE *, int, int);
 extern enum calling_abi ix86_cfun_abi (void);
 extern enum calling_abi ix86_function_type_abi (const_tree);
 
+extern bool ix86_use_pseudo_pic_reg (void);
+
+extern void ix86_reset_previous_fndecl (void);
+
 #ifdef RTX_CODE
 extern int standard_80387_constant_p (rtx);
 extern const char *standard_80387_constant_opcode (rtx);
 extern rtx standard_80387_constant_rtx (int);
 extern int standard_sse_constant_p (rtx);
-extern const char *standard_sse_constant_opcode (rtx, rtx);
+extern const char *standard_sse_constant_opcode (rtx_insn *, rtx);
 extern bool symbolic_reference_mentioned_p (rtx);
 extern bool extended_reg_mentioned_p (rtx);
-extern bool x86_extended_QIreg_mentioned_p (rtx);
+extern bool x86_extended_QIreg_mentioned_p (rtx_insn *);
 extern bool x86_extended_reg_mentioned_p (rtx);
-extern bool x86_maybe_negate_const_int (rtx *, enum machine_mode);
-extern enum machine_mode ix86_cc_mode (enum rtx_code, rtx, rtx);
+extern bool x86_maybe_negate_const_int (rtx *, machine_mode);
+extern machine_mode ix86_cc_mode (enum rtx_code, rtx, rtx);
 
-extern int avx_vpermilp_parallel (rtx par, enum machine_mode mode);
-extern int avx_vperm2f128_parallel (rtx par, enum machine_mode mode);
+extern int avx_vpermilp_parallel (rtx par, machine_mode mode);
+extern int avx_vperm2f128_parallel (rtx par, machine_mode mode);
 
-extern bool ix86_expand_movmem (rtx, rtx, rtx, rtx, rtx, rtx);
-extern bool ix86_expand_setmem (rtx, rtx, rtx, rtx, rtx, rtx);
 extern bool ix86_expand_strlen (rtx, rtx, rtx, rtx);
+extern bool ix86_expand_set_or_movmem (rtx, rtx, rtx, rtx, rtx, rtx,
+				       rtx, rtx, rtx, rtx, bool);
 
 extern bool constant_address_p (rtx);
 extern bool legitimate_pic_operand_p (rtx);
 extern bool legitimate_pic_address_disp_p (rtx);
-extern bool ix86_legitimize_reload_address (rtx, enum machine_mode,
+extern bool ix86_legitimize_reload_address (rtx, machine_mode,
 					    int, int, int);
 extern void print_reg (rtx, int, FILE*);
 extern void ix86_print_operand (FILE *, rtx, int);
 
-extern void split_double_mode (enum machine_mode, rtx[], int, rtx[], rtx[]);
+extern void split_double_mode (machine_mode, rtx[], int, rtx[], rtx[]);
 
 extern const char *output_set_got (rtx, rtx);
 extern const char *output_387_binary_op (rtx, rtx*);
 extern const char *output_387_reg_move (rtx, rtx*);
-extern const char *output_fix_trunc (rtx, rtx*, bool);
+extern const char *output_fix_trunc (rtx_insn *, rtx*, bool);
 extern const char *output_fp_compare (rtx, rtx*, bool, bool);
 extern const char *output_adjust_stack_and_probe (rtx);
 extern const char *output_probe_stack_range (rtx, rtx);
 
 extern void ix86_expand_clear (rtx);
-extern void ix86_expand_move (enum machine_mode, rtx[]);
-extern void ix86_expand_vector_move (enum machine_mode, rtx[]);
-extern void ix86_expand_vector_move_misalign (enum machine_mode, rtx[]);
-extern void ix86_expand_push (enum machine_mode, rtx);
+extern void ix86_expand_move (machine_mode, rtx[]);
+extern void ix86_expand_vector_move (machine_mode, rtx[]);
+extern void ix86_expand_vector_move_misalign (machine_mode, rtx[]);
 extern rtx ix86_fixup_binary_operands (enum rtx_code,
-				       enum machine_mode, rtx[]);
+				       machine_mode, rtx[]);
 extern void ix86_fixup_binary_operands_no_copy (enum rtx_code,
-						enum machine_mode, rtx[]);
+						machine_mode, rtx[]);
 extern void ix86_expand_binary_operator (enum rtx_code,
-					 enum machine_mode, rtx[]);
+					 machine_mode, rtx[]);
 extern void ix86_expand_vector_logical_operator (enum rtx_code,
-						 enum machine_mode, rtx[]);
-extern bool ix86_binary_operator_ok (enum rtx_code, enum machine_mode, rtx[]);
-extern bool ix86_avoid_lea_for_add (rtx, rtx[]);
-extern bool ix86_use_lea_for_mov (rtx, rtx[]);
-extern bool ix86_avoid_lea_for_addr (rtx, rtx[]);
-extern void ix86_split_lea_for_addr (rtx, rtx[], enum machine_mode);
-extern bool ix86_lea_for_add_ok (rtx, rtx[]);
+						 machine_mode, rtx[]);
+extern bool ix86_binary_operator_ok (enum rtx_code, machine_mode, rtx[]);
+extern bool ix86_avoid_lea_for_add (rtx_insn *, rtx[]);
+extern bool ix86_use_lea_for_mov (rtx_insn *, rtx[]);
+extern bool ix86_avoid_lea_for_addr (rtx_insn *, rtx[]);
+extern void ix86_split_lea_for_addr (rtx_insn *, rtx[], machine_mode);
+extern bool ix86_lea_for_add_ok (rtx_insn *, rtx[]);
 extern bool ix86_vec_interleave_v2df_operator_ok (rtx operands[3], bool high);
 extern bool ix86_dep_by_shift_count (const_rtx set_insn, const_rtx use_insn);
-extern bool ix86_agi_dependent (rtx set_insn, rtx use_insn);
-extern void ix86_expand_unary_operator (enum rtx_code, enum machine_mode,
+extern bool ix86_agi_dependent (rtx_insn *set_insn, rtx_insn *use_insn);
+extern void ix86_expand_unary_operator (enum rtx_code, machine_mode,
 					rtx[]);
-extern rtx ix86_build_const_vector (enum machine_mode, bool, rtx);
-extern rtx ix86_build_signbit_mask (enum machine_mode, bool, bool);
+extern rtx ix86_build_const_vector (machine_mode, bool, rtx);
+extern rtx ix86_build_signbit_mask (machine_mode, bool, bool);
 extern void ix86_split_convert_uns_si_sse (rtx[]);
 extern void ix86_expand_convert_uns_didf_sse (rtx, rtx);
 extern void ix86_expand_convert_uns_sixf_sse (rtx, rtx);
@@ -113,13 +116,13 @@ extern void ix86_expand_convert_sign_didf_sse (rtx, rtx);
 extern void ix86_expand_vector_convert_uns_vsivsf (rtx, rtx);
 extern rtx ix86_expand_adjust_ufix_to_sfix_si (rtx, rtx *);
 extern enum ix86_fpcmp_strategy ix86_fp_comparison_strategy (enum rtx_code);
-extern void ix86_expand_fp_absneg_operator (enum rtx_code, enum machine_mode,
+extern void ix86_expand_fp_absneg_operator (enum rtx_code, machine_mode,
 					    rtx[]);
 extern void ix86_expand_copysign (rtx []);
 extern void ix86_split_copysign_const (rtx []);
 extern void ix86_split_copysign_var (rtx []);
-extern bool ix86_unary_operator_ok (enum rtx_code, enum machine_mode, rtx[]);
-extern bool ix86_match_ccmode (rtx, enum machine_mode);
+extern bool ix86_unary_operator_ok (enum rtx_code, machine_mode, rtx[]);
+extern bool ix86_match_ccmode (rtx, machine_mode);
 extern void ix86_expand_branch (enum rtx_code, rtx, rtx, rtx);
 extern void ix86_expand_setcc (rtx, enum rtx_code, rtx, rtx);
 extern bool ix86_expand_int_movcc (rtx[]);
@@ -135,45 +138,36 @@ extern void ix86_split_call_vzeroupper (rtx, rtx);
 extern void x86_initialize_trampoline (rtx, rtx, rtx);
 extern rtx ix86_zero_extend_to_Pmode (rtx);
 extern void ix86_split_long_move (rtx[]);
-extern void ix86_split_ashl (rtx *, rtx, enum machine_mode);
-extern void ix86_split_ashr (rtx *, rtx, enum machine_mode);
-extern void ix86_split_lshr (rtx *, rtx, enum machine_mode);
+extern void ix86_split_ashl (rtx *, rtx, machine_mode);
+extern void ix86_split_ashr (rtx *, rtx, machine_mode);
+extern void ix86_split_lshr (rtx *, rtx, machine_mode);
 extern rtx ix86_find_base_term (rtx);
 extern bool ix86_check_movabs (rtx, int);
-extern void ix86_split_idivmod (enum machine_mode, rtx[], bool);
+extern void ix86_split_idivmod (machine_mode, rtx[], bool);
 
-extern rtx assign_386_stack_local (enum machine_mode, enum ix86_stack_slot);
-extern int ix86_attr_length_immediate_default (rtx, bool);
-extern int ix86_attr_length_address_default (rtx);
-extern int ix86_attr_length_vex_default (rtx, bool, bool);
+extern rtx assign_386_stack_local (machine_mode, enum ix86_stack_slot);
+extern int ix86_attr_length_immediate_default (rtx_insn *, bool);
+extern int ix86_attr_length_address_default (rtx_insn *);
+extern int ix86_attr_length_vex_default (rtx_insn *, bool, bool);
 
-extern enum machine_mode ix86_fp_compare_mode (enum rtx_code);
+extern machine_mode ix86_fp_compare_mode (enum rtx_code);
 
-extern rtx ix86_libcall_value (enum machine_mode);
+extern rtx ix86_libcall_value (machine_mode);
 extern bool ix86_function_arg_regno_p (int);
 extern void ix86_asm_output_function_label (FILE *, const char *, tree);
-extern rtx ix86_force_to_memory (enum machine_mode, rtx);
-extern void ix86_free_from_memory (enum machine_mode);
 extern void ix86_call_abi_override (const_tree);
 extern int ix86_reg_parm_stack_space (const_tree);
 
 extern void ix86_split_fp_branch (enum rtx_code code, rtx, rtx,
-				  rtx, rtx, rtx, rtx);
-extern bool ix86_hard_regno_mode_ok (int, enum machine_mode);
-extern bool ix86_modes_tieable_p (enum machine_mode, enum machine_mode);
+				  rtx, rtx, rtx);
+extern bool ix86_hard_regno_mode_ok (int, machine_mode);
+extern bool ix86_modes_tieable_p (machine_mode, machine_mode);
 extern bool ix86_secondary_memory_needed (enum reg_class, enum reg_class,
-					  enum machine_mode, int);
-extern bool ix86_cannot_change_mode_class (enum machine_mode,
-					   enum machine_mode, enum reg_class);
+					  machine_mode, int);
+extern bool ix86_cannot_change_mode_class (machine_mode,
+					   machine_mode, enum reg_class);
 
-extern int ix86_mode_needed (int, rtx);
-extern int ix86_mode_after (int, int, rtx);
-extern int ix86_mode_entry (int);
-extern int ix86_mode_exit (int);
-
-#ifdef HARD_CONST
-extern void ix86_emit_mode_set (int, int, HARD_REG_SET);
-#endif
+extern bool ix86_libc_has_function (enum function_class fn_class);
 
 extern void x86_order_regs_for_local_alloc (void);
 extern void x86_function_profiler (FILE *, int);
@@ -182,10 +176,10 @@ extern void ix86_emit_fp_unordered_jump (rtx);
 
 extern void ix86_emit_i387_log1p (rtx, rtx);
 extern void ix86_emit_i387_round (rtx, rtx);
-extern void ix86_emit_swdivsf (rtx, rtx, rtx, enum machine_mode);
-extern void ix86_emit_swsqrtsf (rtx, rtx, enum machine_mode, bool);
+extern void ix86_emit_swdivsf (rtx, rtx, rtx, machine_mode);
+extern void ix86_emit_swsqrtsf (rtx, rtx, machine_mode, bool);
 
-extern enum rtx_code ix86_reverse_condition (enum rtx_code, enum machine_mode);
+extern enum rtx_code ix86_reverse_condition (enum rtx_code, machine_mode);
 
 extern void ix86_expand_lround (rtx, rtx);
 extern void ix86_expand_lfloorceil (rtx, rtx, bool);
@@ -207,16 +201,18 @@ extern void init_cumulative_args (CUMULATIVE_ARGS *, tree, rtx, tree, int);
 #endif	/* RTX_CODE  */
 
 #ifdef TREE_CODE
-extern int ix86_data_alignment (tree, int);
-extern unsigned int ix86_local_alignment (tree, enum machine_mode,
+extern int ix86_data_alignment (tree, int, bool);
+extern unsigned int ix86_local_alignment (tree, machine_mode,
 					  unsigned int);
-extern unsigned int ix86_minimum_alignment (tree, enum machine_mode,
+extern unsigned int ix86_minimum_alignment (tree, machine_mode,
 					    unsigned int);
 extern int ix86_constant_alignment (tree, int);
 extern tree ix86_handle_shared_attribute (tree *, tree, tree, int, bool *);
 extern tree ix86_handle_selectany_attribute (tree *, tree, tree, int, bool *);
 extern int x86_field_alignment (tree, int);
-extern tree ix86_valid_target_attribute_tree (tree);
+extern tree ix86_valid_target_attribute_tree (tree,
+					      struct gcc_options *,
+					      struct gcc_options *);
 extern unsigned int ix86_get_callcvt (const_tree);
 
 #endif
@@ -234,6 +230,9 @@ extern void ix86_expand_mul_widen_evenodd (rtx, rtx, rtx, bool, bool);
 extern void ix86_expand_mul_widen_hilo (rtx, rtx, rtx, bool, bool);
 extern void ix86_expand_sse2_mulv4si3 (rtx, rtx, rtx);
 extern void ix86_expand_sse2_mulvxdi3 (rtx, rtx, rtx);
+extern void ix86_expand_sse2_abs (rtx, rtx);
+
+extern bool ix86_bnd_prefixed_insn_p (rtx);
 
 /* In i386-c.c  */
 extern void ix86_target_macros (void);
@@ -260,10 +259,11 @@ extern void i386_pe_end_function (FILE *, const char *, tree);
 extern void i386_pe_assemble_visibility (tree, int);
 extern tree i386_pe_mangle_decl_assembler_name (tree, tree);
 extern tree i386_pe_mangle_assembler_name (const char *);
+extern void i386_pe_record_stub (const char *);
 
 extern void i386_pe_seh_init (FILE *);
 extern void i386_pe_seh_end_prologue (FILE *);
-extern void i386_pe_seh_unwind_emit (FILE *, rtx);
+extern void i386_pe_seh_unwind_emit (FILE *, rtx_insn *);
 extern void i386_pe_seh_emit_except_personality (rtx);
 extern void i386_pe_seh_init_sections (void);
 
@@ -305,7 +305,7 @@ extern int asm_preferred_eh_data_format (int, int);
 extern enum attr_cpu ix86_schedule;
 #endif
 
-extern const char * ix86_output_call_insn (rtx insn, rtx call_op);
+extern const char * ix86_output_call_insn (rtx_insn *insn, rtx call_op);
 
 #ifdef RTX_CODE
 /* Target data for multipass lookahead scheduling.
