@@ -1,7 +1,7 @@
 /* Prototypes of target machine functions, Lattice Mico32 architecture.
    Contributed by Jon Beniston <jon@beniston.com>
 
-   Copyright (C) 2009-2014 Free Software Foundation, Inc.
+   Copyright (C) 2009-2015 Free Software Foundation, Inc.
 
    This file is part of GCC.
 
@@ -32,7 +32,7 @@ extern int lm32_can_use_return (void);
 extern rtx lm32_return_addr_rtx (int count, rtx frame);
 extern int lm32_expand_block_move (rtx *);
 extern int nonpic_symbol_mentioned_p (rtx);
-extern rtx lm32_legitimize_pic_address (rtx, enum machine_mode, rtx);
+extern rtx lm32_legitimize_pic_address (rtx, machine_mode, rtx);
 extern void lm32_expand_scc (rtx operands[]);
 extern void lm32_expand_conditional_branch (rtx operands[]);
-extern bool lm32_move_ok (enum machine_mode, rtx operands[2]);
+extern bool lm32_move_ok (machine_mode, rtx operands[2]);
