@@ -92,8 +92,8 @@ along with GCC; see the file COPYING3.  If not see
    is examined for still-unused equivalence conditions.  We create a
    block for each merged equivalence list.  */
 
+#include <map>
 #include "config.h"
-#define INCLUDE_MAP
 #include "system.h"
 #include "coretypes.h"
 #include "tm.h"
@@ -810,21 +810,13 @@ element_number (gfc_array_ref *ar)
       if (ar->dimen_type[i] != DIMEN_ELEMENT)
         gfc_internal_error ("element_number(): Bad dimension type");
 
-      if (as && as->lower[i])
-	mpz_sub (n, *get_mpz (ar->start[i]), *get_mpz (as->lower[i]));
-      else
-	mpz_sub_ui (n, *get_mpz (ar->start[i]), 1);
+      mpz_sub (n, *get_mpz (ar->start[i]), *get_mpz (as->lower[i]));
  
       mpz_mul (n, n, multiplier);
       mpz_add (offset, offset, n);
  
-      if (as && as->upper[i] && as->lower[i])
-	{
-	  mpz_sub (extent, *get_mpz (as->upper[i]), *get_mpz (as->lower[i]));
-	  mpz_add_ui (extent, extent, 1);
-	}
-      else
-	mpz_set_ui (extent, 0);
+      mpz_sub (extent, *get_mpz (as->upper[i]), *get_mpz (as->lower[i]));
+      mpz_add_ui (extent, extent, 1);
  
       if (mpz_sgn (extent) < 0)
         mpz_set_ui (extent, 0);
@@ -1247,12 +1239,8 @@ finish_equivalences (gfc_namespace *ns)
 	  {
 	    c = gfc_get_common_head ();
 	    /* We've lost the real location, so use the location of the
-	       enclosing procedure.  If we're in a BLOCK DATA block, then
-	       use the location in the sym_root.  */
-	    if (ns->proc_name)
-	      c->where = ns->proc_name->declared_at;
-	    else if (ns->is_block_data)
-	      c->where = ns->sym_root->n.sym->declared_at;
+	       enclosing procedure.  */
+	    c->where = ns->proc_name->declared_at;
 	    strcpy (c->name, z->module);
 	  }
 	else
