@@ -1,5 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009, 2010
-   Free Software Foundation
+/* Copyright (C) 2001-2015 Free Software Foundation, Inc.
 
    This file is part of libgcj.
 
@@ -13,7 +12,19 @@ details.  */
 /* #define VERIFY_DEBUG */
 
 #include "config.h"
+#include "system.h"
+#include "coretypes.h"
 
+#include "hash-set.h"
+#include "machmode.h"
+#include "vec.h"
+#include "double-int.h"
+#include "input.h"
+#include "alias.h"
+#include "symtab.h"
+#include "options.h"
+#include "wide-int.h"
+#include "inchash.h"
 #include "verify.h"
 
 /* Hack to work around namespace pollution from java-tree.h.  */
