@@ -2,7 +2,7 @@
 /* { dg-do compile } */
 /* { dg-options "-O -fstack-check=generic" } */
 
-static inline bar (char *c, int i)
+static inline int bar (char *c, int i)
 {
   return c + i > c;
 }
