@@ -3,8 +3,8 @@
 /* { dg-options "-O -Wuninitialized" } */
 
 /* Not all platforms support TImode integers.  */
-#if defined(__LP64__) || defined(__sparc__)
-typedef int TItype __attribute__ ((mode (TI)));  /* { dg-error "no data type for mode" "TI" { target sparc-sun-solaris2.[0-6] sparc-sun-solaris2.[0-6].* } } */
+#if (defined(__LP64__) && !defined(__hppa__)) || defined(__SPU__)
+typedef int TItype __attribute__ ((mode (TI)));
 #else
 typedef long TItype;
 #endif
