@@ -1,7 +1,6 @@
 /* { dg-require-effective-target vect_int } */
 
 #include <stdarg.h>
-#include <stdio.h>
 #include "tree-vect.h"
 
 #define N 16 
@@ -9,10 +8,10 @@
 unsigned int out[N];
 unsigned int in[N] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
 
+int i;
 __attribute__ ((noinline)) int
 main1 ()
 {
-  int i;
   unsigned int *pin = &in[0];
   unsigned int *pout = &out[0];
   
@@ -21,6 +20,9 @@ main1 ()
   *pout++ = *pin++;
   *pout++ = *pin++;
 
+  if (i)
+    __asm__ volatile ("" : : : "memory");
+
   /* Check results.  */
   if (out[0] != in[0]
       || out[1] != in[1]
@@ -40,6 +42,6 @@ int main (void)
   return 0;
 }
 
-/* { dg-final { scan-tree-dump-times "basic block vectorized using SLP" 1 "slp" } } */
-/* { dg-final { cleanup-tree-dump "slp" } } */
+/* { dg-final { scan-tree-dump-times "basic block vectorized" 1 "slp2" } } */
+/* { dg-final { cleanup-tree-dump "slp2" } } */
   
