@@ -9,6 +9,7 @@
 #define SIZE (AVX512F_LEN / 32)
 #include "avx512f-mask-type.h"
 
+static void
 CALC (MASK_TYPE *r, int *s1, int *s2)
 {
   int i;
@@ -20,13 +21,12 @@ CALC (MASK_TYPE *r, int *s1, int *s2)
       *r = *r | (one << i);
 }
 
-void static
+void
 TEST (void)
 {
   int i;
   UNION_TYPE (AVX512F_LEN, i_d) src1, src2;
-  MASK_TYPE res_ref, res1;
-  MASK_TYPE mask = MASK_VALUE;
+  MASK_TYPE res_ref, res1, res2;
   res1 = 0;
 
   for (i = 0; i < SIZE / 2; i++)
@@ -38,9 +38,15 @@ TEST (void)
     }
 
   res1 = INTRINSIC (_cmple_epi32_mask) (src1.x, src2.x);
+  res2 = INTRINSIC (_mask_cmple_epi32_mask) (MASK_VALUE, src1.x, src2.x);
 
   CALC (&res_ref, src1.a, src2.a);
 
   if (res_ref != res1)
     abort ();
+
+  res_ref &= MASK_VALUE;
+
+  if (res_ref != res2)
+    abort ();
 }
