@@ -11,6 +11,8 @@
 
 #include "go-alloc.h"
 #include "runtime.h"
+#include "arch.h"
+#include "malloc.h"
 
 /* Set the C environment from Go.  This is called by syscall.Setenv.  */
 
@@ -23,6 +25,7 @@ setenv_c (String k, String v)
   unsigned char *kn;
   const byte *vs;
   unsigned char *vn;
+  intgo len;
 
   ks = k.str;
   if (ks == NULL)
@@ -38,14 +41,22 @@ setenv_c (String k, String v)
 
   if (ks != NULL && ks[k.len] != 0)
     {
-      kn = __go_alloc (k.len + 1);
+      // Objects that are explicitly freed must be at least 16 bytes in size,
+      // so that they are not allocated using tiny alloc.
+      len = k.len + 1;
+      if (len < TinySize)
+	len = TinySize;
+      kn = __go_alloc (len);
       __builtin_memcpy (kn, ks, k.len);
       ks = kn;
     }
 
   if (vs != NULL && vs[v.len] != 0)
     {
-      vn = __go_alloc (v.len + 1);
+      len = v.len + 1;
+      if (len < TinySize)
+	len = TinySize;
+      vn = __go_alloc (len);
       __builtin_memcpy (vn, vs, v.len);
       vs = vn;
     }
@@ -54,7 +65,10 @@ setenv_c (String k, String v)
 
 #else /* !defined(HAVE_SETENV) */
 
-  kn = __go_alloc (k.len + v.len + 2);
+  len = k.len + v.len + 2;
+  if (len < TinySize)
+    len = TinySize;
+  kn = __go_alloc (len);
   __builtin_memcpy (kn, ks, k.len);
   kn[k.len] = '=';
   __builtin_memcpy (kn + k.len + 1, vs, v.len);
