@@ -1,6 +1,6 @@
 // Locale support -*- C++ -*-
 
-// Copyright (C) 2000-2014 Free Software Foundation, Inc.
+// Copyright (C) 2000-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -60,7 +60,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	{
 #if defined (_CTYPE_S) || defined (__istype)
 	  *__vec = __maskrune (*__low, upper | lower | alpha | digit | xdigit
-			       | space | print | graph | cntrl | punct | alnum);
+			       | space | print | graph | cntrl | punct | alnum
+			       | blank);
 #else
 	  mask __m = 0;
 	  if (this->is(upper, *__low)) __m |= upper;
@@ -75,6 +76,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	  if (this->is(punct, *__low)) __m |= punct;
 	  // Do not include explicit line for alnum mask since it is a
 	  // pure composite of masks on FreeBSD.
+	  if (this->is(blank, *__low)) __m |= blank;
 	  *__vec = __m;
 #endif
 	}
@@ -123,7 +125,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
   {
     for (; __lo < __hi; ++__vec, ++__lo)
       *__vec = __maskrune (*__lo, upper | lower | alpha | digit | xdigit
-			   | space | print | graph | cntrl | punct | alnum);
+			   | space | print | graph | cntrl | punct | alnum
+			   | blank);
     return __hi;
   }
   
