@@ -1,6 +1,6 @@
 // regex -*- C++ -*-
 
-// Copyright (C) 2011-2013 Free Software Foundation, Inc.
+// Copyright (C) 2011-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -22,7 +22,8 @@
 // see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
 // <http://www.gnu.org/licenses/>.
 
-#include <regex>
+#include <stdexcept>
+#include <bits/regex_error.h>
 
 namespace std _GLIBCXX_VISIBILITY(default)
 {
@@ -34,12 +35,5 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 
   regex_error::~regex_error() throw() { }
 
-#ifdef _AIX
-  // PR libstdc++/52887
-  template class function<void (__regex::_PatternCursor const&,
-				__regex::_Results&)>;
-  template class function<bool (__regex::_PatternCursor const&)>;
-#endif
-
 _GLIBCXX_END_NAMESPACE_VERSION
 } // namespace std
