@@ -1,4 +1,4 @@
-// Copyright (C) 2003-2014 Free Software Foundation, Inc.
+// Copyright (C) 2003-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -22,42 +22,40 @@
 #include <locale>
 #include <sstream>
 #include <ctime>
+#include <cstring>
 #include <testsuite_hooks.h>
 
-class TP : public std::time_put<wchar_t>
-{
-public:
-  mutable std::string format_chars;
-
-protected:
-  iter_type do_put(iter_type s, std::ios_base&, char_type,
-		   const std::tm*, char format, char) const
-  {
-    format_chars.push_back(format);
-    return s;
-  }
-};
-
 // libstdc++/12439
-// time_put::put reads past end of format string
-void test03()
+// time_put::put writes narrowed characters to output iterator
+void test02()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
+
+  typedef time_put<wchar_t> tp_type;
   
-  wostringstream stream;
+  const wchar_t fmt[] = {
+    0xa0, 0x103, 0xfc, 0xb3, 0xa0c3,
+    L'%', L'c'
+  };
+  
+  const size_t len = sizeof(fmt) / sizeof(fmt[0]);
+  const size_t cmplen = wcschr(fmt, L'%') - fmt;
+  
+  locale loc;
+  const tp_type& tp = use_facet<tp_type>(loc);
   time_t tt = time(0);
+  wostringstream stream;
   
-  const wchar_t* fmt = L"%c";
-  
-  TP tp;
-  tp.put(TP::iter_type(stream), stream, stream.fill(), localtime(&tt),
-	 fmt, fmt + 1);
-  VERIFY( tp.format_chars.empty() );
+  tp.put(tp_type::iter_type(stream), stream, stream.fill(),
+	 localtime(&tt), fmt, fmt + len);
+  wstring str = stream.str();
+  VERIFY( str.length() >= cmplen );
+  VERIFY( !wmemcmp(str.data(), fmt, cmplen) );
 }
 
 int main()
 {
-  test03();
+  test02();
   return 0;
 }
