@@ -61,11 +61,11 @@ void test01()
   locale loc(locale::classic(), cvt);
 
   wfilebuf fb;
+  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
   fb.sputc(L'a');
-  fb.pubsync(); // Does not call unshift()
 
   VERIFY( !cvt->unshift_called );
   fb.pubseekoff(0, ios_base::cur);
