@@ -61,12 +61,12 @@ void test01()
   locale loc(locale::classic(), cvt);
 
   wfilebuf fb;
-  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
   streampos pos = fb.pubseekoff(0, ios_base::beg);
   fb.sputc(L'a');
+  fb.pubsync(); // Does not call unshift()
 
   VERIFY( !cvt->unshift_called );
   fb.pubseekpos(pos);
