@@ -1,6 +1,6 @@
 // 2000-06-29 bkoz
 
-// Copyright (C) 2000-2014 Free Software Foundation, Inc.
+// Copyright (C) 2000-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -34,12 +34,12 @@ void test06(void)
   // seekg
   {
     istringstream iss(num1);
-    iss.tellg();
+    istream::pos_type pos1 = iss.tellg();
     int asNum = 0;
     iss >> asNum;
     VERIFY( test = iss.eof() );
     VERIFY( test = !iss.fail() );
-    iss.seekg(0, ios_base::beg);
+    iss.seekg(pos1);
     VERIFY( test = !iss.fail() );
   }
 }
