@@ -1,6 +1,6 @@
 // 2006-10-12  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2006-2013 Free Software Foundation, Inc.
+// Copyright (C) 2006-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -17,18 +17,18 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-// { dg-options "-DWIDTH=500000" { target simulator } }
-
 // 27.6.2.5.4 basic_ostream character inserters
 
+// { dg-options "-DMAX_SIZE=50000" { target simulator } }
+
+#ifndef MAX_SIZE
+#define MAX_SIZE 5000000
+#endif
+
 #include <ostream>
 #include <sstream>
 #include <testsuite_hooks.h>
 
-#ifndef WIDTH
-#define WIDTH 50000000
-#endif
-
 // libstdc++/28277
 void test01()
 {
@@ -36,11 +36,12 @@ void test01()
   bool test __attribute__((unused)) = true;
 
   wostringstream oss_01;
+  const string str_01(50, 'a');
 
-  oss_01.width(WIDTH);
+  oss_01.width(MAX_SIZE);
   const streamsize width = oss_01.width();
 
-  oss_01 << L'a';
+  oss_01 << str_01.c_str();
 
   VERIFY( oss_01.good() );
   VERIFY( oss_01.str().size() == wstring::size_type(width) );
