@@ -19,20 +19,19 @@
 
 #include <iostream>
 
-// libstdc++/2523
-void test03()
+// libstdc++/6648
+// Interactive tests: each one (run alone) must terminate upon a single '\n'.
+void test08()
 {
-  using namespace std;
-  ios_base::sync_with_stdio(false);
-
-  int i;
-  wcin >> i;
-  wcout << "i == " << i << endl;
+  bool test __attribute__((unused)) = true;
+  wchar_t buff[2048];
+  std::wcout << "Enter name: ";
+  std::wcin.getline(buff, 2048);
 }
 
 int 
 main()
 {
-  test03();
+  test08();
   return 0;
 }
